-- Migration script from ART 1.11rev2 or 1.11rev3 to ART 2.0
--
-- Purpose: create/update the tables needed to 
--          . support olap and formatted reports
--          . support default object groups and can change password option for users
--          . streamline database schema
--
-- this script will work as is for mysql, oracle, postgresql, hsqldb, sql server
--
-- If you are running Art version less than 1.11rev2/3, pls. download
-- Art 1.11rev3 and upgrade to it before attempting to upgrade to 2.0
-- ------------------------------------------------


-- change to support olap and formatted reports
ALTER TABLE ART_QUERIES ADD TEMPLATE VARCHAR(100);
ALTER TABLE ART_QUERIES ADD XMLA_URL VARCHAR(300);
ALTER TABLE ART_QUERIES ADD XMLA_DATASOURCE VARCHAR(50);
ALTER TABLE ART_QUERIES ADD XMLA_CATALOG VARCHAR(50);

-- change to support default object groups and can change password option for users
ALTER TABLE ART_USERS ADD DEFAULT_OBJECT_GROUP INTEGER;
ALTER TABLE ART_USERS ADD CAN_CHANGE_PASSWORD VARCHAR(1);
ALTER TABLE ART_USER_GROUPS ADD DEFAULT_OBJECT_GROUP INTEGER;

-- change to ART_JOBS to support job names
ALTER TABLE ART_JOBS ADD JOB_NAME VARCHAR(50);

-- change to have separate fields for graph options
ALTER TABLE ART_QUERIES ADD X_AXIS_LABEL VARCHAR(2000);
ALTER TABLE ART_QUERIES ADD Y_AXIS_LABEL VARCHAR(50);
ALTER TABLE ART_QUERIES ADD GRAPH_OPTIONS VARCHAR(200);

UPDATE ART_QUERIES SET X_AXIS_LABEL=DESCRIPTION;


-- streamline ART_USERS 
ALTER TABLE ART_USERS ADD ADMIN_LEVEL INTEGER;
ALTER TABLE ART_USERS ADD ACTIVE_STATUS VARCHAR(1);

UPDATE ART_USERS SET ADMIN_LEVEL=CNT1;
UPDATE ART_USERS SET ACTIVE_STATUS=ART_FLAG;

ALTER TABLE ART_USERS DROP COLUMN CNT1;
ALTER TABLE ART_USERS DROP COLUMN ART_FLAG;

ALTER TABLE ART_USERS DROP COLUMN LOCATION;
ALTER TABLE ART_USERS DROP COLUMN COUNTRY;
ALTER TABLE ART_USERS DROP COLUMN NICK_NAME;


-- streamline ART_QUERIES 
ALTER TABLE ART_QUERIES ADD QUERY_TYPE INTEGER;
ALTER TABLE ART_QUERIES ADD CONTACT_PERSON VARCHAR(20);
ALTER TABLE ART_QUERIES ADD ACTIVE_STATUS VARCHAR(1);
ALTER TABLE ART_QUERIES ADD USES_RULES VARCHAR(1);

UPDATE ART_QUERIES SET QUERY_TYPE=CNT1;
UPDATE ART_QUERIES SET CONTACT_PERSON=OTH1;
UPDATE ART_QUERIES SET ACTIVE_STATUS=ART_FLAG;
UPDATE ART_QUERIES SET USES_RULES=SMART_RULES_FLAG;

ALTER TABLE ART_QUERIES DROP COLUMN CNT1;
ALTER TABLE ART_QUERIES DROP COLUMN OTH1;
ALTER TABLE ART_QUERIES DROP COLUMN ART_FLAG;
ALTER TABLE ART_QUERIES DROP COLUMN SMART_RULES_FLAG;

-- streamline ART_QUERY_FIELDS
ALTER TABLE ART_QUERY_FIELDS DROP COLUMN OTH1;
ALTER TABLE ART_QUERY_FIELDS DROP COLUMN ART_FLAG;
ALTER TABLE ART_QUERY_FIELDS DROP COLUMN QUERY_GROUP_ID;

ALTER TABLE ART_QUERY_FIELDS ADD USE_LOV VARCHAR(1);
ALTER TABLE ART_QUERY_FIELDS ADD PARAM_TYPE VARCHAR(1);
ALTER TABLE ART_QUERY_FIELDS ADD PARAM_LABEL VARCHAR(55);
ALTER TABLE ART_QUERY_FIELDS ADD APPLY_RULES_TO_LOV VARCHAR(1);
ALTER TABLE ART_QUERY_FIELDS ADD LOV_QUERY_ID INTEGER;
ALTER TABLE ART_QUERY_FIELDS ADD CHAINED_PARAM_POSITION INTEGER;

UPDATE ART_QUERY_FIELDS SET USE_LOV=SQL_RETRIEVE_FLAG;
UPDATE ART_QUERY_FIELDS SET PARAM_TYPE=MULTIPLE_SELECTION_FLAG;
UPDATE ART_QUERY_FIELDS SET PARAM_LABEL=MULTIPLE_FIELD_NAME;
UPDATE ART_QUERY_FIELDS SET APPLY_RULES_TO_LOV=SMART_RULES_FLAG;
UPDATE ART_QUERY_FIELDS SET LOV_QUERY_ID=SQL_STATEMENT_QUERY_ID;
UPDATE ART_QUERY_FIELDS SET CHAINED_PARAM_POSITION=CNT1;

ALTER TABLE ART_QUERY_FIELDS DROP COLUMN SQL_RETRIEVE_FLAG;
ALTER TABLE ART_QUERY_FIELDS DROP COLUMN MULTIPLE_SELECTION_FLAG;
ALTER TABLE ART_QUERY_FIELDS DROP COLUMN MULTIPLE_FIELD_NAME;
ALTER TABLE ART_QUERY_FIELDS DROP COLUMN SMART_RULES_FLAG;
ALTER TABLE ART_QUERY_FIELDS DROP COLUMN SQL_STATEMENT_QUERY_ID;
ALTER TABLE ART_QUERY_FIELDS DROP COLUMN CNT1;

UPDATE ART_QUERY_FIELDS SET PARAM_TYPE='M' WHERE PARAM_TYPE='Y';

-- streamline ART_USER_RULES
ALTER TABLE ART_USER_RULES ADD RULE_VALUE VARCHAR(25);
ALTER TABLE ART_USER_RULES ADD RULE_TYPE VARCHAR(6);

UPDATE ART_USER_RULES SET RULE_VALUE=VALUE;
UPDATE ART_USER_RULES SET RULE_TYPE=TYPE;

ALTER TABLE ART_USER_RULES DROP COLUMN VALUE;
ALTER TABLE ART_USER_RULES DROP COLUMN TYPE;

-- streamline ART_DATABASES
ALTER TABLE ART_DATABASES ADD POOL_TIMEOUT INTEGER;
ALTER TABLE ART_DATABASES ADD TEST_SQL VARCHAR(20);

UPDATE ART_DATABASES SET POOL_TIMEOUT=CNT1;
UPDATE ART_DATABASES SET TEST_SQL=OTH1;

ALTER TABLE ART_DATABASES DROP COLUMN CNT1;
ALTER TABLE ART_DATABASES DROP COLUMN OTH1;

-- streamline ART_JOBS
ALTER TABLE ART_JOBS ADD JOB_TYPE INTEGER;
ALTER TABLE ART_JOBS ADD ACTIVE_STATUS VARCHAR(1);
ALTER TABLE ART_JOBS ADD ENABLE_AUDIT VARCHAR(1);

UPDATE ART_JOBS SET JOB_TYPE=TYPE;
UPDATE ART_JOBS SET ACTIVE_STATUS=ART_FLAG;
UPDATE ART_JOBS SET ENABLE_AUDIT=AUDIT_FLAG;

ALTER TABLE ART_JOBS DROP COLUMN TYPE;
ALTER TABLE ART_JOBS DROP COLUMN ART_FLAG;
ALTER TABLE ART_JOBS DROP COLUMN AUDIT_FLAG;

UPDATE ART_JOBS SET ACTIVE_STATUS='A' WHERE ACTIVE_STATUS='Y';
UPDATE ART_JOBS SET ACTIVE_STATUS='D' WHERE ACTIVE_STATUS='N';

-- streamline ART_JOBS_PARAMETERS
ALTER TABLE ART_JOBS_PARAMETERS ADD PARAM_TYPE CHAR(1);
ALTER TABLE ART_JOBS_PARAMETERS ADD PARAM_NAME VARCHAR(60);
ALTER TABLE ART_JOBS_PARAMETERS ADD PARAM_VALUE VARCHAR(60);

UPDATE ART_JOBS_PARAMETERS SET PARAM_TYPE=TYPE;
UPDATE ART_JOBS_PARAMETERS SET PARAM_NAME=NAME;
UPDATE ART_JOBS_PARAMETERS SET PARAM_VALUE=VALUE;

ALTER TABLE ART_JOBS_PARAMETERS DROP COLUMN TYPE;
ALTER TABLE ART_JOBS_PARAMETERS DROP COLUMN NAME;
ALTER TABLE ART_JOBS_PARAMETERS DROP COLUMN VALUE;

-- streamline ART_LOGS
ALTER TABLE ART_LOGS ADD LOG_TYPE VARCHAR(15);

UPDATE ART_LOGS SET LOG_TYPE=TYPE;

ALTER TABLE ART_LOGS DROP COLUMN TYPE;


