-- Upgrade script from ART 3.7 to ART 3.8

-- CHANGES:
-- update database version
-- make some varchar columns nullable
-- create roles and permissions tables


-- NOTES:
-- for sql server, mysql, replace TIMESTAMP with DATETIME
-- ------------------------------------------------


-- update database version
UPDATE ART_DATABASE_VERSION SET DATABASE_VERSION='3.8';

-- insert custom upgrade record
INSERT INTO ART_CUSTOM_UPGRADES VALUES('3.8', 0);

-- make some varchar columns nullable
ALTER TABLE ART_QUERY_GROUPS ADD DESCRIPTION_TEMP VARCHAR(200);
UPDATE ART_QUERY_GROUPS SET DESCRIPTION_TEMP=DESCRIPTION;
ALTER TABLE ART_QUERY_GROUPS DROP COLUMN DESCRIPTION;
ALTER TABLE ART_QUERY_GROUPS ADD DESCRIPTION VARCHAR(200);
UPDATE ART_QUERY_GROUPS SET DESCRIPTION=DESCRIPTION_TEMP;
ALTER TABLE ART_QUERY_GROUPS DROP COLUMN DESCRIPTION_TEMP;

-- create roles and permissions tables
UPDATE ART_USERS SET ACCESS_LEVEL=0 WHERE ACCESS_LEVEL IS NULL;

-- ART_ROLES
-- Stores roles

CREATE TABLE ART_ROLES
(
	ROLE_ID INTEGER NOT NULL,
	NAME VARCHAR(50) NOT NULL,
	DESCRIPTION VARCHAR(200),
	CREATION_DATE TIMESTAMP,
	CREATED_BY VARCHAR(50),
	UPDATE_DATE TIMESTAMP,
	UPDATED_BY VARCHAR(50),
	CONSTRAINT art_rls_pk PRIMARY KEY(ROLE_ID),
	CONSTRAINT art_rls_uq_nm UNIQUE(NAME)
);

-- create roles for upgrade
INSERT INTO ART_ROLES (ROLE_ID, NAME) VALUES(1, 'Normal User');
INSERT INTO ART_ROLES (ROLE_ID, NAME) VALUES(2, 'Schedule User');
INSERT INTO ART_ROLES (ROLE_ID, NAME) VALUES(3, 'Junior Admin');
INSERT INTO ART_ROLES (ROLE_ID, NAME) VALUES(4, 'Mid Admin');
INSERT INTO ART_ROLES (ROLE_ID, NAME) VALUES(5, 'Standard Admin');
INSERT INTO ART_ROLES (ROLE_ID, NAME) VALUES(6, 'Senior Admin');
INSERT INTO ART_ROLES (ROLE_ID, NAME) VALUES(7, 'Super Admin');


-- ART_PERMISSIONS
-- Stores permissions

CREATE TABLE ART_PERMISSIONS
(
	PERMISSION_ID INTEGER NOT NULL,
	NAME VARCHAR(100) NOT NULL,
	CONSTRAINT art_pm_pk PRIMARY KEY(PERMISSION_ID),
	CONSTRAINT art_pm_uq_nm UNIQUE(NAME)
);

-- insert permissions
INSERT INTO ART_PERMISSIONS VALUES(1, 'view_reports');
INSERT INTO ART_PERMISSIONS VALUES(2, 'save_reports');
INSERT INTO ART_PERMISSIONS VALUES(3, 'self_service_dashboards');
INSERT INTO ART_PERMISSIONS VALUES(4, 'schedule_jobs');
INSERT INTO ART_PERMISSIONS VALUES(5, 'view_jobs');
INSERT INTO ART_PERMISSIONS VALUES(6, 'configure_jobs');
INSERT INTO ART_PERMISSIONS VALUES(7, 'view_logs');
INSERT INTO ART_PERMISSIONS VALUES(8, 'configure_users');
INSERT INTO ART_PERMISSIONS VALUES(9, 'configure_art_database');
INSERT INTO ART_PERMISSIONS VALUES(10, 'configure_settings');
INSERT INTO ART_PERMISSIONS VALUES(11, 'configure_user_groups');
INSERT INTO ART_PERMISSIONS VALUES(12, 'configure_datasources');
INSERT INTO ART_PERMISSIONS VALUES(13, 'configure_reports');
INSERT INTO ART_PERMISSIONS VALUES(14, 'configure_caches');
INSERT INTO ART_PERMISSIONS VALUES(15, 'configure_connections');
INSERT INTO ART_PERMISSIONS VALUES(16, 'configure_loggers');
INSERT INTO ART_PERMISSIONS VALUES(17, 'configure_report_groups');
INSERT INTO ART_PERMISSIONS VALUES(18, 'configure_schedules');
INSERT INTO ART_PERMISSIONS VALUES(19, 'configure_holidays');
INSERT INTO ART_PERMISSIONS VALUES(20, 'configure_destinations');
INSERT INTO ART_PERMISSIONS VALUES(21, 'configure_admin_rights');
INSERT INTO ART_PERMISSIONS VALUES(22, 'configure_access_rights');
INSERT INTO ART_PERMISSIONS VALUES(23, 'configure_user_group_membership');
INSERT INTO ART_PERMISSIONS VALUES(24, 'configure_report_group_membership');
INSERT INTO ART_PERMISSIONS VALUES(25, 'configure_smtp_servers');
INSERT INTO ART_PERMISSIONS VALUES(26, 'configure_encryptors');
INSERT INTO ART_PERMISSIONS VALUES(27, 'migrate_records');
INSERT INTO ART_PERMISSIONS VALUES(28, 'configure_roles');
INSERT INTO ART_PERMISSIONS VALUES(29, 'configure_permissions');


-- ART_ROLE_PERMISSION_MAP
-- Stores role - permission mappings

CREATE TABLE ART_ROLE_PERMISSION_MAP
(
	ROLE_ID INTEGER NOT NULL,
	PERMISSION_ID INTEGER NOT NULL,
	CONSTRAINT art_rpm_pk PRIMARY KEY(ROLE_ID, PERMISSION_ID)
);

-- create role permissions for upgrade
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(1, 1);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(1, 2);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(1, 5);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(2, 1);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(2, 2);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(2, 4);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(2, 5);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(3, 1);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(3, 2);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(3, 3);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(3, 4);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(3, 5);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(3, 13);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(4, 1);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(4, 2);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(4, 3);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(4, 4);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(4, 5);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(4, 13);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(4, 22);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(5, 1);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(5, 2);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(5, 3);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(5, 4);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(5, 5);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(5, 6);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(5, 8);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(5, 11);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(5, 13);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(5, 21);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(5, 22);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(5, 23);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(5, 24);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 1);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 2);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 3);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 4);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 5);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 6);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 7);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 8);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 11);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 12);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 13);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 14);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 15);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 16);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 17);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 18);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 19);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 20);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 21);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 22);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 23);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 24);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 25);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 26);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 27);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 28);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(6, 29);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 1);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 2);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 3);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 4);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 5);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 6);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 7);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 8);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 9);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 10);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 11);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 12);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 13);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 14);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 15);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 16);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 17);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 18);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 19);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 20);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 21);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 22);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 23);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 24);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 25);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 26);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 27);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 28);
INSERT INTO ART_ROLE_PERMISSION_MAP VALUES(7, 29);


-- ART_USER_ROLE_MAP
-- Stores user - role mappings

CREATE TABLE ART_USER_ROLE_MAP
(
	USER_ID INTEGER NOT NULL,
	ROLE_ID INTEGER NOT NULL,
	CONSTRAINT art_urm_pk PRIMARY KEY(USER_ID, ROLE_ID)
);


-- ART_USER_GROUP_ROLE_MAP
-- Stores user group - role mappings

CREATE TABLE ART_USER_GROUP_ROLE_MAP
(
	USER_GROUP_ID INTEGER NOT NULL,
	ROLE_ID INTEGER NOT NULL,
	CONSTRAINT art_ugrm_pk PRIMARY KEY(USER_GROUP_ID, ROLE_ID)
);


-- ART_USER_PERMISSION_MAP
-- Stores user - permission mappings

CREATE TABLE ART_USER_PERMISSION_MAP
(
	USER_ID INTEGER NOT NULL,
	PERMISSION_ID INTEGER NOT NULL,
	CONSTRAINT art_upm_pk PRIMARY KEY(USER_ID, PERMISSION_ID)
);


-- ART_USER_GROUP_PERM_MAP
-- Stores user group - permission mappings

CREATE TABLE ART_USER_GROUP_PERM_MAP
(
	USER_GROUP_ID INTEGER NOT NULL,
	PERMISSION_ID INTEGER NOT NULL,
	CONSTRAINT art_ugpm_pk PRIMARY KEY(USER_GROUP_ID, PERMISSION_ID)
);
