/*
 * ART. A Reporting Tool.
 * Copyright (C) 2017 Enrico Liboni <eliboni@users.sf.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package art.general;

import java.util.ArrayList;
import java.util.List;
import org.owasp.encoder.Encode;

/**
 * Represents results of an action
 *
 * @author Timothy Anyona
 */
public class ActionResult {

	private boolean success;
	private String message; //i18n message
	private Object data;

	/**
	 * @return the success
	 */
	public boolean isSuccess() {
		return success;
	}

	/**
	 * @param success the success to set
	 */
	public void setSuccess(boolean success) {
		this.success = success;
	}

	/**
	 * @return the message
	 */
	public String getMessage() {
		return message;
	}

	/**
	 * @param message the message to set
	 */
	public void setMessage(String message) {
		this.message = message;
	}

	/**
	 * @return the data
	 */
	public Object getData() {
		return data;
	}

	/**
	 * @param data the data to set
	 */
	public void setData(Object data) {
		this.data = data;
	}

	/**
	 * Cleans the result data for html output. The data is expected to be
	 * a List of String.
	 *
	 * @return the cleaned data
	 */
	public List<String> cleanData() {
		@SuppressWarnings("unchecked")
		List<String> rawData = (List<String>) data;
		List<String> cleanedData = new ArrayList<>();
		for (String item : rawData) {
			String cleanedItem = Encode.forHtmlContent(item);
			cleanedData.add(cleanedItem);
		}

		return cleanedData;
	}

}
