/*
 * ====================================================================
 * This software is subject to the terms of the Common Public License
 * Agreement, available at the following URL:
 *   http://www.opensource.org/licenses/cpl.html .
 * Copyright (C) 2003-2004 TONBELLER AG.
 * All Rights Reserved.
 * You must accept the terms of that agreement to use this software.
 * ====================================================================
 *
 * 
 */
package net.sf.jpivotart.jpivot.mondrian;

import net.sf.mondrianart.mondrian.olap.Util.PropertyList;
import net.sf.mondrianart.mondrian.rolap.RolapConnectionProperties;

import net.sf.jpivotart.jpivot.core.ExtensionSupport;
import net.sf.jpivotart.jpivot.olap.model.Cell;
import net.sf.jpivotart.jpivot.olap.navi.DrillThrough;
import net.sf.wcfart.wcf.table.TableModel;
/**
 * @author Robin Bagot
 *
 * Implementation of the DrillExpand Extension for Mondrian Data Source.
*/
public class MondrianDrillThrough extends ExtensionSupport implements DrillThrough {

  private boolean extendedContext = true;

  /**
   * Constructor sets ID
   */
  public MondrianDrillThrough() {
    super.setId(DrillThrough.ID);
  }

  /**
   * drill through is possible if <code>member</code> is not calculated
   */
  public boolean canDrillThrough(Cell cell) {
    return ((MondrianCell) cell).getMonCell().canDrillThrough();
    //String sql = ((MondrianCell) cell).getMonCell().getDrillThroughSQL(extendedContext);
    //return sql != null;
  }

  /**
   * does a drill through, retrieves data that makes up the selected Cell
   */
  public TableModel drillThrough(Cell cell) {
    String sql = ((MondrianCell) cell).getMonCell().getDrillThroughSQL(extendedContext);
    if (sql == null) {
      throw new NullPointerException("DrillThroughSQL returned null");
    }
    MondrianDrillThroughTableModel dtm = new MondrianDrillThroughTableModel();
    dtm.setSql(sql);
    PropertyList connectInfo = ((MondrianModel) getModel()).getConnectProperties();
    String jdbcUrl = connectInfo.get(RolapConnectionProperties.Jdbc.name());
    String jdbcUser = connectInfo.get(RolapConnectionProperties.JdbcUser.name());
    String jdbcPassword = connectInfo.get(RolapConnectionProperties.JdbcPassword.name());
    String dataSourceName = connectInfo.get(RolapConnectionProperties.DataSource.name());
    dtm.setJdbcUrl(jdbcUrl);
    dtm.setJdbcUser(jdbcUser);
    dtm.setJdbcPassword(jdbcPassword);
    dtm.setDataSourceName(dataSourceName);
    dtm.setExternalDataSource(((MondrianModel)getModel()).getExternalDataSource());
    return dtm;
  }

  public boolean isExtendedContext() {
    return extendedContext;
  }

  public void setExtendedContext(boolean extendedContext) {
    this.extendedContext = extendedContext;
  }

}
