/*
 *   Copyright 2014 OSBI Ltd
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
package org.saiku.service.util;

import java.util.Map;

public final class KeyValue<K, V> implements Map.Entry<K, V> {
  private final K key;
  private V value;

  public KeyValue( K key, V value ) {
    this.key = key;
    this.value = value;
  }

  public K getKey() {
    return key;
  }

  public V getValue() {
    return value;
  }

  public V setValue( V value ) {
    V old = this.value;
    this.value = value;
    return old;
  }

  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ( ( key == null ) ? 0 : key.hashCode() );
    return result;
  }

  @SuppressWarnings("unchecked")
  @Override
  public boolean equals( Object obj ) {
    if ( this == obj ) {
      return true;
    }
    if ( obj == null ) {
      return false;
    }
    if ( getClass() != obj.getClass() ) {
      return false;
    }
    KeyValue<K, V> other = (KeyValue<K, V>) obj;
    if ( key == null ) {
      if ( other.key != null ) {
        return false;
      }
    } else if ( !key.equals( other.key ) ) {
      return false;
    }
    return true;
  }


}
